# # import app
import optparse
import os
import sys

sys.path.insert(0, os.getcwd())

from chalice_babel import Babel

from app import app

babel = Babel(app)


from chalice import Chalice


try:
    # See: https://setuptools.pypa.io/en/latest/deprecated/distutils-legacy.html
    from setuptools import Command as _Command

    try:
        from setuptools.errors import BaseError, OptionError, SetupError
    except ImportError:  # Error aliases only added in setuptools 59 (2021-11).
        OptionError = SetupError = BaseError = Exception

except ImportError:
    from distutils.cmd import Command as _Command
    from distutils.errors import DistutilsError as BaseError
    from distutils.errors import DistutilsSetupError as SetupError
    from distutils.errors import OptionError as OptionError


class Command(_Command):
    as_args = None

    multiple_value_options = ()
    boolean_options = ()
    option_aliases = {}
    option_choices = {}

    def __init__(self, dist=None):
        self.distribution = dist
        self.initialize_options()
        self._dry_run = None
        self.verbose = False
        self.force = None
        self.help = 0
        self.finalized = 0


class export_strings(Command):

    description = "export translation strings from .po files"
    user_options = [
        ("lang=", "l", "lang"),
        ("domain", "D", "domain"),
        ("translation_folder", "t", "translation folder"),
        ("output_dir", "o", "output dir"),
        ("filename", "f", "filename"),
    ]
    boolean_options = ["no-wrap"]

    def initialize_options(self):
        self.lang = "en"
        self.domain = "messages"
        self.output_dir = None
        self.translation_folder = "translations"
        self.filename = "translation_strings"

    def finalize_options(self):
        pass
        # raise OptionError('you must specify the input file')

    def run(self):

        babel.export_strings(lang=self.lang, domain=self.domain, translation_folder= self.translation_folder, output_dir=self.output_dir, filename=self.filename)


class import_strings(Command):

    description = "import translation strings from json file to .po files"
    user_options = [
        ("domain", "D", "domain"),
        ("translation_folder", "t", "translation folder"),
        ("input_dir", "i", "input dir"),
        ("filename", "f", "filename"),
    ]
    boolean_options = ["no-wrap"]

    def initialize_options(self):
        self.domain = "messages"
        self.translation_folder = "translations"
        self.input_dir = None
        self.filename = "translation_strings"

    def finalize_options(self):
        pass

    def run(self):
        babel.import_strings(domain=self.domain, translation_folder=self.translation_folder, input_dir=self.input_dir, filename=self.filename)


class CommandLineInterface(object):

    usage = "%%prog %s [options] %s"
    version = "%%prog %s" % "0.4"
    commands = {
        "import_strings": "import translations from json file to .po files",
        "export_strings": "exports translations to json from .po files",
    }

    command_classes = {
        "import_strings": import_strings,
        "export_strings": export_strings,
    }

    def run(self, argv=None):

        if argv is None:
            argv = sys.argv

        self.parser = optparse.OptionParser(usage=self.usage % ("command", "[args]"), version=self.version)
        self.parser.disable_interspersed_args()
        self.parser.print_help = self._help

        options, args = self.parser.parse_args(argv[1:])

        if not args:
            self.parser.error("no valid command or option passed. " "Try the -h/--help option for more information.")

        cmdname = args[0]
        if cmdname not in self.commands:
            self.parser.error('unknown command "%s"' % cmdname)

        cmdinst = self._configure_command(cmdname, args[1:])
        return cmdinst.run()

    def _help(self):
        print(self.parser.format_help())
        print("commands:")
        longest = max([len(command) for command in self.commands])
        format = "  %%-%ds %%s" % max(8, longest + 1)
        commands = sorted(self.commands.items())
        for name, description in commands:
            print(format % (name, description))

    def _configure_command(self, cmdname, argv):

        cmdclass = self.command_classes[cmdname]
        cmdinst = cmdclass()
        assert isinstance(cmdinst, Command)
        cmdinst.initialize_options()

        parser = optparse.OptionParser(usage=self.usage % (cmdname, ""), description=self.commands[cmdname])
        as_args = getattr(cmdclass, "as_args", ())
        for long, short, help in cmdclass.user_options:
            name = long.strip("=")
            default = getattr(cmdinst, name.replace("-", "_"))
            strs = ["--%s" % name]
            if short:
                strs.append("-%s" % short)
            strs.extend(cmdclass.option_aliases.get(name, ()))
            choices = cmdclass.option_choices.get(name, None)
            if name == as_args:
                parser.usage += "<%s>" % name
            elif name in cmdclass.boolean_options:
                parser.add_option(*strs, action="store_true", help=help)
            elif name in cmdclass.multiple_value_options:
                parser.add_option(*strs, action="append", help=help, choices=choices)
            else:
                parser.add_option(*strs, help=help, default=default, choices=choices)
        options, args = parser.parse_args(argv)

        if as_args:
            setattr(options, as_args.replace("-", "_"), args)

        for key, value in vars(options).items():
            setattr(cmdinst, key, value)

        try:
            cmdinst.ensure_finalized()
        except OptionError as err:
            parser.error(str(err))

        return cmdinst


def main():
    return CommandLineInterface().run(sys.argv)


if __name__ == "__main__":
    main()



import os

from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), "README.md")) as readme:
    README = readme.read()


setup(
    name="chalice_babel",
    version="1.0.0",
    author="Sufle",
    author_email="info@sufle.io",
    description="Chalice Babel Package",
    long_description_content_type="text/markdown",
    long_description=README,
    url="https://github.com/sufleio/chalice-babel",
    license="MIT",
    packages=find_packages(),
    python_requires=">=3.6",
    install_requires=[
        "Babel>=2.10.3",
        "chalice>=1.26.6",
        "Jinja2>=3.1.2",
        "pytz>=2022.5",
    ],
    extras_require={
        "dev": [
            "pytest",
            "pytest-mock",
        ]
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    zip_safe=False,
    entry_points="""
    [console_scripts]
    chalice_babel = chalice_babel.command.main:main
    
    [distutils.commands]
    import = chalice_babel.command.main:import_strings
    export_strings = chalice_babel.command.main:export_strings

    """,
)



import json
import os
import re
from contextlib import contextmanager
from datetime import datetime
from io import StringIO

from functools import cached_property
from chalice_babel.lazy_string import LazyString

from babel.messages.pofile import Catalog, read_po, write_po

# TODO: Exception Fix
try:
    import app as current_app
except ModuleNotFoundError:
    pass

from babel import Locale, dates, numbers, support
from jinja2 import Environment, FileSystemLoader, select_autoescape
from pytz import UTC, timezone
from werkzeug.datastructures import ImmutableDict

accept_language_re = "([A-Za-z]{1,8}(?:-[A-Za-z0-9]{1,8})*|\*)(?:\s*;\s*q=(0(?:\.[0-9]{,3})?|1(?:\.0{,3})?))?(?:\s*,\s*|$)"  # noqa: W605


class Babel(object):

    default_date_formats = ImmutableDict(
        {
            "time": "medium",
            "date": "medium",
            "datetime": "medium",
            "time.short": None,
            "time.medium": None,
            "time.full": None,
            "time.long": None,
            "date.short": None,
            "date.medium": None,
            "date.full": None,
            "date.long": None,
            "datetime.short": None,
            "datetime.medium": None,
            "datetime.full": None,
            "datetime.long": None,
        }
    )

    def __init__(
        self,
        app,
        default_locale="en",
        default_timezone="UTC",
        default_domain="messages",
        config=None,
        date_formats=None,
        configure_jinja=True,
    ):
        self.app = app
        self._default_locale = default_locale
        self._default_timezone = default_timezone
        self._default_domain = default_domain
        self._date_formats = date_formats
        self.locale_selector_func = None
        self.timezone_selector_func = None
        self.config = self.config_file()
        self._configure_jinja = configure_jinja

        if not hasattr(app, "chalice_babel"):
            app.chalice_babel = {}
        app.chalice_babel["babel"] = self
        if self._date_formats is None:
            self._date_formats = self.default_date_formats.copy()

        self.date_formats = self._date_formats

        if self._configure_jinja:

            env = Environment(
                loader=FileSystemLoader(os.getcwd() + "/chalicelib/templates"),
                autoescape=select_autoescape(),
                extensions=["jinja2.ext.i18n"],
            )
            app.chalice_babel["jinja2_env"] = env
            app.chalice_babel["jinja2_env"].filters.update(
                datetimeformat=format_datetime,
                dateformat=format_date,
                timeformat=format_time,
                timedeltaformat=format_timedelta,
                numberformat=format_number,
                decimalformat=format_decimal,
                currencyformat=format_currency,
                percentformat=format_percent,
                scientificformat=format_scientific,
            )
            app.chalice_babel["jinja2_env"].install_gettext_callables(
                lambda x: get_translations().ugettext(x),
                lambda s, p, n: get_translations().ungettext(s, p, n),
                newstyle=True,
            )

    def config_file(self):
        try:
            from chalicelib import babel_config

            config = babel_config
        except KeyError:
            config = {
                "BABEL_DEFAULT_LOCALE": self._default_locale,
                "BABEL_DEFAULT_TIMEZONE": self.default_timezone,
            }
        return config

    def localeselector(self, f):
        self.locale_selector_func = f
        return f

    def timezoneselector(self, f):
        self.timezone_selector_func = f
        return f

    def parse_language_header(self):
        request = self.app.current_request
        langs = request.headers["accept-language"]
        return langs

    def best_match(self, lang_string):
        result = []

        result = []
        pieces = re.match(f"{accept_language_re}", lang_string.lower(), re.VERBOSE)
        request_lang = pieces.groups()[0]
        config = self.app.chalice_babel["babel"].config
        supported_langs = config.get("LANGUAGES", self._default_locale)
        if request_lang in supported_langs:
            return request_lang
        return self._default_locale

    @property
    def default_locale(self):

        config = self.app.chalice_babel["babel"].config
        babel_default_locale = config.get("BABEL_DEFAULT_LOCALE", self._default_locale)
        return Locale.parse(babel_default_locale)

    @property
    def default_timezone(self):

        config = self.app.chalice_babel["babel"].config
        babel_default_timezone = config.get(
            "BABEL_DEFAULT_TIMEZONE", self._default_timezone
        )
        return timezone(babel_default_timezone)

    @property
    def domain(self):

        config = self.app.chalice_babel["babel"].config
        babel_domain = config.get("BABEL_DOMAIN", self._default_domain)
        return babel_domain

    @cached_property
    def domain_instance(self):

        config = self.app.chalice_babel["babel"].config
        babel_domain = config.get("BABEL_DOMAIN", self._default_domain)
        return Domain(domain=babel_domain)

    def list_translations(self):

        result = []

        for dirname in self.translation_directories:
            if not os.path.isdir(dirname):
                continue

            for folder in os.listdir(dirname):
                locale_dir = os.path.join(dirname, folder, "LC_MESSAGES")
                if not os.path.isdir(locale_dir):
                    continue

                if any(x.endswith(".mo") for x in os.listdir(locale_dir)):
                    result.append(Locale.parse(folder))

        if not result:
            result.append(Locale.parse(self._default_locale))

        return result

    @property
    def translation_directories(self):
        config = self.app.chalice_babel["babel"].config
        directories = config.get("BABEL_TRANSLATION_DIRECTORIES", "translations")
        for path in directories:
            if os.path.isabs(path):
                yield path
            else:
                yield os.path.join(os.getcwd() + "/chalicelib/", path)

    def export_strings(
        self,
        lang="en",
        domain="messages",
        translation_folder="translations",
        output_dir=None,
        filename="translation_strings",
    ):
        translations = os.path.join(os.getcwd() + "/chalicelib/", translation_folder)
        po_path = f"{translations}/{lang}/LC_MESSAGES/{domain}.po"
        source_str = StringIO(open(po_path, "r", encoding="utf-8").read())
        source_catalog = read_po(source_str)
        for_tron = {
            message.id: {lang: message.string}
            for message in source_catalog
            if message.id
        }

        for locale in self.list_translations():
            locale = locale.language
            if locale != lang:
                target_str = StringIO(
                    open(
                        translations + "/" + locale + "/LC_MESSAGES/" + domain + ".po",
                        "r",
                        encoding="utf-8",
                    ).read()
                )
                target_catalog = read_po(target_str)

                for message in target_catalog:
                    if message.id and message.id in for_tron.keys():
                        for_tron[message.id][locale] = message.string

        if not output_dir:
            translation_file = os.getcwd() + "/" + filename + ".json"
            with open(translation_file, "w", encoding="utf-8") as outfile:
                json.dump(for_tron, outfile, ensure_ascii=False, indent=4)

        else:
            translation_file = os.getcwd() + "/" + output_dir + "/" + filename + ".json"
            if not os.path.exists(os.path.dirname(translation_file)):
                try:
                    os.makedirs(os.path.dirname(translation_file))
                except OSError as exc:
                    if exc.errno != errno.EEXIST:
                        raise
            with open(translation_file, "w", encoding="utf-8") as outfile:
                json.dump(for_tron, outfile, ensure_ascii=False, indent=4)

    def import_strings(
        self,
        filename="translation_strings",
        translation_folder="translations",
        input_dir=None,
        domain="messages",
    ):
        translations = os.path.join(os.getcwd() + "/chalicelib/", translation_folder)
        if input_dir:
            from_tron = json.loads(
                open(
                    os.getcwd() + "/" + input_dir + "/" + filename + ".json",
                    "r",
                    encoding="utf-8",
                ).read()
            )
        else:
            from_tron = json.loads(
                open(
                    os.getcwd() + "/" + filename + ".json", "r", encoding="utf-8"
                ).read()
            )

        template_str = StringIO(open(f"{domain}.pot", "r", encoding="utf-8").read())
        template = read_po(template_str)

        for locale in self.list_translations():
            locale = locale.language
            new_catalog = Catalog(fuzzy=False)
            for id in from_tron:
                if locale in from_tron[id].keys():
                    new_catalog.add(id, from_tron[id][locale])
            new_catalog.update(template)
            po_path = f"{translations}/{locale}/LC_MESSAGES/{domain}.po"
            write_po(open(po_path, "wb"), new_catalog)


@contextmanager
def force_locale(locale):
    ctx = current_app.app.chalice_babel
    ctx["babel_locale"] = getattr(ctx, "babel_locale", None)
    orig_attrs = {}
    orig_attrs["babel_locale"] = getattr(ctx, "babel_locale", None)

    try:
        ctx["babel_locale"] = Locale.parse(locale)
        ctx["forced_babel_locale"] = ctx["babel_locale"]
        yield
    finally:
        if "forced_babel_locale" in ctx:
            del ctx["forced_babel_locale"]

        for key, value in orig_attrs.items():
            ctx[key] = value


def get_locale():

    babel = current_app.app.chalice_babel["babel"]
    locale = current_app.app.chalice_babel.get("babel_locale", None)
    if locale is None:
        if babel.locale_selector_func is None:
            locale = babel.default_locale
        else:
            rv = babel.locale_selector_func()
            if rv is None:
                locale = babel.default_locale
            else:
                locale = Locale.parse(rv)
    return locale


def get_timezone():

    babel = current_app.app.chalice_babel["babel"]
    if babel.timezone_selector_func is None:
        tzinfo = babel.default_timezone
    else:
        rv = babel.timezone_selector_func()
        if rv is None:
            tzinfo = babel.default_timezone
        else:
            tzinfo = timezone(rv) if isinstance(rv, str) else rv
    return tzinfo


def format_decimal(number, format=None):

    locale = get_locale()
    return numbers.format_decimal(number, format=format, locale=locale)


def format_number(number):

    locale = get_locale()
    return numbers.format_decimal(number, locale=locale)


def format_currency(
    number, currency, format=None, currency_digits=True, format_type="standard"
):

    locale = get_locale()
    return numbers.format_currency(
        number,
        currency,
        format=format,
        locale=locale,
        currency_digits=currency_digits,
        format_type=format_type,
    )


def format_percent(number, format=None):

    locale = get_locale()
    return numbers.format_percent(number, format=format, locale=locale)


def format_scientific(number, format=None):

    locale = get_locale()
    return numbers.format_scientific(number, format=format, locale=locale)


def get_translations():

    return get_domain().get_translations()


def _get_format(key, format):
    babel = current_app.app.chalice_babel["babel"]
    if format is None:
        format = babel.date_formats[key]
    if format in ("short", "medium", "full", "long"):
        rv = babel.date_formats["%s.%s" % (key, format)]
        if rv is not None:
            format = rv
    return format


def to_user_timezone(datetime):
    if datetime.tzinfo is None:
        datetime = datetime.replace(tzinfo=UTC)
    tzinfo = get_timezone()
    return tzinfo.normalize(datetime.astimezone(tzinfo))


def to_utc(datetime):
    if datetime.tzinfo is None:
        datetime = get_timezone().localize(datetime)
    return datetime.astimezone(UTC).replace(tzinfo=None)


def format_datetime(datetime=None, format=None, rebase=True):
    format = _get_format("datetime", format)
    return _date_format(dates.format_datetime, datetime, format, rebase)


def format_date(date=None, format=None, rebase=True):
    if rebase and isinstance(date, datetime):
        date = to_user_timezone(date)
    format = _get_format("date", format)
    return _date_format(dates.format_date, date, format, rebase)


def format_time(time=None, format=None, rebase=True):
    format = _get_format("time", format)
    return _date_format(dates.format_time, time, format, rebase)


def format_timedelta(
    datetime_or_timedelta, granularity="second", add_direction=False, threshold=0.85
):

    if isinstance(datetime_or_timedelta, datetime):
        datetime_or_timedelta = datetime.utcnow() - datetime_or_timedelta
    return dates.format_timedelta(
        datetime_or_timedelta,
        granularity,
        threshold=threshold,
        add_direction=add_direction,
        locale=get_locale(),
    )


def _date_format(formatter, obj, format, rebase, **extra):
    locale = get_locale()
    extra = {}
    if formatter is not dates.format_date and rebase:
        extra["tzinfo"] = get_timezone()
    return formatter(obj, format, locale=locale, **extra)


class Domain:
    def __init__(self, translation_directories=None, domain="messages"):
        if isinstance(translation_directories, str):
            translation_directories = [translation_directories]
        self._translation_directories = translation_directories
        self.domain = domain
        self.cache = {}

    def __repr__(self):
        return "<Domain({!r}, {!r})>".format(self._translation_directories, self.domain)

    @property
    def translation_directories(self):

        if self._translation_directories is not None:
            return self._translation_directories
        babel = current_app.app.chalice_babel["babel"]
        return babel.translation_directories

    def get_translations_cache(self):
        return self.cache

    def get_translations(self):
        cache = self.get_translations_cache()
        locale = get_locale()
        try:
            return cache[str(locale), self.domain]
        except KeyError:
            translations = support.Translations()

            for dirname in self.translation_directories:
                catalog = support.Translations.load(dirname, [locale], self.domain)
                translations.merge(catalog)
                if hasattr(catalog, "plural"):
                    translations.plural = catalog.plural

            cache[str(locale), self.domain] = translations
            return translations

    def gettext(self, string, **variables):

        t = self.get_translations()
        s = t.ugettext(string)
        return s if not variables else s % variables

    def ngettext(self, singular, plural, num, **variables):

        variables.setdefault("num", num)
        t = self.get_translations()
        s = t.ungettext(singular, plural, num)
        return s if not variables else s % variables

    def pgettext(self, context, string, **variables):

        t = self.get_translations()
        s = t.upgettext(context, string)
        return s if not variables else s % variables

    def npgettext(self, context, singular, plural, num, **variables):

        variables.setdefault("num", num)
        t = self.get_translations()
        s = t.unpgettext(context, singular, plural, num)
        return s if not variables else s % variables

    def lazy_gettext(self, string, **variables):

        return LazyString(self.gettext, string, **variables)

    def lazy_ngettext(self, singular, plural, num, **variables):

        return LazyString(self.ngettext, singular, plural, num, **variables)


def get_domain():
    babel = current_app.app.chalice_babel["babel"]
    babel_domain = babel.domain_instance
    return babel_domain


def gettext(*args, **kwargs):
    return get_domain().gettext(*args, **kwargs)


_ = gettext


def ngettext(*args, **kwargs):
    return get_domain().ngettext(*args, **kwargs)


def pgettext(*args, **kwargs):
    return get_domain().pgettext(*args, **kwargs)


def lazy_gettext(*args, **kwargs):
    return LazyString(gettext, *args, **kwargs)


def lazy_ngettext(*args, **kwargs):
    return LazyString(ngettext, *args, **kwargs)


def render_template(template_name, context={}):

    jinja2_env = current_app.app.chalice_babel["jinja2_env"]
    template = jinja2_env.get_or_select_template(template_name)
    return template.render(context)


def render_template_string(source, **context):

    jinja2_env = current_app.app.chalice_babel["jinja2_env"]
    template = jinja2_env.from_string(source)
    return template.render(context)
